@@ -33,7 +33,7 @@
 #include "nfslib.h"
 
 static void	closedown(int sig);
-static int	makesock(int port, int proto);
+int	makesock(int port, int proto);
 
 #define _RPCSVC_CLOSEDOWN	120
 int	_rpcpmstart = 0;
@@ -127,7 +127,6 @@ int sig;
 {
 	(void) signal(sig, closedown);
 	if (_rpcsvcdirty == 0) {
-		extern fd_set svc_fdset;
 		static int size;
 		int i, openfd;
 
@@ -145,9 +144,7 @@ int sig;
 	(void) alarm(_RPCSVC_CLOSEDOWN);
 }
 
-static int makesock(port, proto)
-int port;
-int proto;
+int makesock(int port, int proto)
 {
 	struct sockaddr_in sin;
 	int	s;
